@@ -1,7 +1,7 @@
-/************************************************************** 
-* Copyright (c) 2011, Dynamic Network Services, Inc.
-* Jake Montgomery (jmontgomery@dyn.com) & Tom Daly (tom@dyn.com)
-* Distributed under the FreeBSD License - see LICENSE
+/**************************************************************
+* Copyright (c) 2013, Dynamic Network Services, Inc. Jake Montgomery
+* (jmontgomery@dyn.com) & Tom Daly (tom@dyn.com) Distributed under the FreeBSD
+* License - see LICENSE
 ***************************************************************/
 #include "common.h"
 #include "SockAddr.h"
@@ -10,66 +10,36 @@
 #include <net/if.h>
 #include <cstring>
 
-using namespace std;
 namespace openbfdd
 {
 
-  Addr::Type Addr::FamilyToType(int af)
-  {
-    if (af == AF_INET)
-      return Addr::IPv4;
-    else if (af == AF_INET6) 
-      return Addr::IPv6;
-    else
-      return Addr::Invalid;
-  }
-
-  int Addr::TypeToFamily(Addr::Type type)
-  {
-    if (type == Addr::IPv4)
-      return AF_INET;
-    else if (Addr::IPv6) 
-      return AF_INET6;
-    else
-      return AF_UNSPEC;
-  }
-
-  const char *Addr::TypeToString(Addr::Type type)
-  {
-    if (type == Addr::IPv4)
-      return "IPv4";
-    else if (Addr::IPv6) 
-      return "IPv6";
-    else
-      return "<unknown>";
-  }
-
   /**
    * Sets the address to invalid, and m_addr to the default for the given type.
-   * 
-   * @param type 
+   *
+   * @param type
    */
   void sockAddrBase::init(Addr::Type type)
   {
-    m_isValid = false;
     switch (type)
     {
-      default:
-      case Addr::IPv4:
-      {
-        sockaddr_in *storage = getIPv4Storage();
-        storage->sin_family = AF_INET;
-        storage->sin_port = 0;
-        storage->sin_addr.s_addr = INADDR_ANY;
-      }
+    case Addr::IPv4:
+    {
+      sockaddr_in *storage = getIPv4Storage();
+      storage->sin_family = AF_INET;
+      storage->sin_port = 0;
+      storage->sin_addr.s_addr = INADDR_ANY;
+    }
       break;
-      case Addr::IPv6:
-      {
-        sockaddr_in6 *storage = getIPv6Storage();
-        *storage = sockaddr_in6();
-        storage->sin6_family = AF_INET6;
-        storage->sin6_addr = in6addr_any;  /** Probably not necessary. Is  in6addr_any _guaranteed_ to be 0?*/
-      }
+    case Addr::IPv6:
+    {
+      sockaddr_in6 *storage = getIPv6Storage();
+      *storage = sockaddr_in6();
+      storage->sin6_family = AF_INET6;
+      storage->sin6_addr = in6addr_any;  /** Probably not necessary. Is  in6addr_any _guaranteed_ to be 0?*/
+    }
+      break;
+    default:
+      m_addr.ss_family = AF_UNSPEC;
       break;
     }
   }
@@ -92,7 +62,6 @@ namespace openbfdd
     memcpy(storage, addr, sizeof(sockaddr_in6));
     if (!m_allowPort)
       clearPort();
-    m_isValid = true;
   }
 
   sockAddrBase::sockAddrBase(bool allowPort, const sockaddr_in *addr) : m_allowPort(allowPort)
@@ -108,7 +77,6 @@ namespace openbfdd
     memcpy(storage, addr, sizeof(sockaddr_in));
     if (!m_allowPort)
       clearPort();
-    m_isValid = true;
   }
 
 
@@ -121,8 +89,7 @@ namespace openbfdd
 
     sockaddr_in6 *storage = getIPv6Storage();
 
-    storage->sin6_addr = *addr; 
-    m_isValid = true;
+    storage->sin6_addr = *addr;
   }
 
 
@@ -135,8 +102,7 @@ namespace openbfdd
 
     sockaddr_in *storage = getIPv4Storage();
 
-    storage->sin_addr.s_addr = addr->s_addr; 
-    m_isValid = true;
+    storage->sin_addr.s_addr = addr->s_addr;
   }
 
   sockAddrBase::sockAddrBase(bool allowPort, Addr::Type type, in_port_t port) : m_allowPort(allowPort)
@@ -172,28 +138,26 @@ namespace openbfdd
     memcpy(&m_addr, addr, GetSize());
     if (!m_allowPort)
       clearPort();
-    m_isValid = true;
-
   }
-  
-  sockAddrBase::sockAddrBase(bool allowPort, const sockAddrBase &src) : m_allowPort(allowPort)  
+
+  sockAddrBase::sockAddrBase(bool allowPort, const sockAddrBase &src) : m_allowPort(allowPort)
   {
     copy(src);
   }
 
-  sockAddrBase::sockAddrBase(bool allowPort, const char *str)  : m_allowPort(allowPort)
+  sockAddrBase::sockAddrBase(bool allowPort, const char *str) : m_allowPort(allowPort)
   {
     FromString(str);
   }
 
 
-  sockAddrBase::sockAddrBase(bool allowPort, const char *str, in_port_t port) : m_allowPort(allowPort) 
+  sockAddrBase::sockAddrBase(bool allowPort, const char *str, in_port_t port) : m_allowPort(allowPort)
   {
     FromString(str, port);
   }
 
 
-  void sockAddrBase::copy(const sockAddrBase& src)
+  void sockAddrBase::copy(const sockAddrBase &src)
   {
     if (!src.IsValid())
     {
@@ -201,8 +165,6 @@ namespace openbfdd
       return;
     }
 
-    m_isValid = src.m_isValid;   
-
     // Note that GetSize will work, because
     // we already made sure src was valid.
     memcpy(&m_addr, &src.m_addr, src.GetSize());
@@ -210,7 +172,7 @@ namespace openbfdd
       clearPort();
   }
 
-  sockAddrBase &sockAddrBase::operator=(const sockAddrBase& src)
+  sockAddrBase& sockAddrBase::operator = (const sockAddrBase &src)
   {
     copy(src);
     return *this;
@@ -221,7 +183,6 @@ namespace openbfdd
     init(type);
     if (type != Addr::Invalid)
     {
-      m_isValid = true;
       if (port != 0 && m_allowPort)
         SetPort(port);
     }
@@ -232,8 +193,28 @@ namespace openbfdd
     if (!IsValid())
       return false;
 
-    if(IsIPv6()) 
-       return IN6_IS_ADDR_UNSPECIFIED(&getIPv6Storage()->sin6_addr);
+    if (IsIPv6())
+    {
+      // Can not use IN6_IS_ADDR_UNSPECIFIED because it breaks strict aliasing on some
+      // platforms?
+      const sockaddr_in6 *storage = getIPv6Storage();
+      return (storage->sin6_addr.s6_addr[0] == 0
+              && storage->sin6_addr.s6_addr[1] == 0
+              && storage->sin6_addr.s6_addr[2] == 0
+              && storage->sin6_addr.s6_addr[3] == 0
+              && storage->sin6_addr.s6_addr[4] == 0
+              && storage->sin6_addr.s6_addr[5] == 0
+              && storage->sin6_addr.s6_addr[6] == 0
+              && storage->sin6_addr.s6_addr[7] == 0
+              && storage->sin6_addr.s6_addr[8] == 0
+              && storage->sin6_addr.s6_addr[9] == 0
+              && storage->sin6_addr.s6_addr[10] == 0
+              && storage->sin6_addr.s6_addr[11] == 0
+              && storage->sin6_addr.s6_addr[12] == 0
+              && storage->sin6_addr.s6_addr[13] == 0
+              && storage->sin6_addr.s6_addr[14] == 0
+              && storage->sin6_addr.s6_addr[15] == 0);
+    }
     else
       return getIPv4Storage()->sin_addr.s_addr == INADDR_ANY;
   }
@@ -260,9 +241,9 @@ namespace openbfdd
   {
     if (IsValid())
     {
-      if(IsIPv6())
+      if (IsIPv6())
         getIPv6Storage()->sin6_port = 0;
-      else 
+      else
         getIPv4Storage()->sin_port  = 0;
     }
   }
@@ -271,9 +252,9 @@ namespace openbfdd
   {
     if (IsValid() && m_allowPort)
     {
-      if(IsIPv6())
+      if (IsIPv6())
         getIPv6Storage()->sin6_port = htons(port);
-      else 
+      else
         getIPv4Storage()->sin_port  = htons(port);
     }
   }
@@ -290,14 +271,20 @@ namespace openbfdd
     init(Addr::Invalid);
   }
 
+
+  bool sockAddrBase::IsValid() const
+  {
+    return (m_addr.ss_family == AF_INET6 || m_addr.ss_family == AF_INET);
+  }
+
   bool sockAddrBase::IsIPv6() const
   {
-    return IsValid() && m_addr.ss_family == AF_INET6;
+    return m_addr.ss_family == AF_INET6;
   }
 
   bool sockAddrBase::IsIPv4() const
   {
-    return IsValid() && m_addr.ss_family == AF_INET;
+    return m_addr.ss_family == AF_INET;
   }
 
   Addr::Type sockAddrBase::Type() const
@@ -306,7 +293,7 @@ namespace openbfdd
     {
       if (m_addr.ss_family == AF_INET)
         return Addr::IPv4;
-      else if (m_addr.ss_family == AF_INET6) 
+      else if (m_addr.ss_family == AF_INET6)
         return Addr::IPv6;
     }
     return Addr::Invalid;
@@ -320,9 +307,9 @@ namespace openbfdd
     if (!IsValid() || !m_allowPort)
       return 0;
 
-    if(IsIPv6())
+    if (IsIPv6())
       port = getIPv6Storage()->sin6_port;
-    else 
+    else
       port = getIPv4Storage()->sin_port;
 
     return ntohs(port);
@@ -344,44 +331,44 @@ namespace openbfdd
 
   socklen_t sockAddrBase::GetSize() const
   {
-    return AddressFamily() == AF_INET6 ? sizeof(sockaddr_in6):sizeof(sockaddr_in);
+    return AddressFamily() == AF_INET6 ? sizeof(sockaddr_in6) : sizeof(sockaddr_in);
   }
 
-  /**
-   * @warning does not checking at all 
-   */
-  inline sockaddr_in6 *sockAddrBase::getIPv6Storage() 
+/**
+ * @warning does no checking at all
+ */
+  inline sockaddr_in6* sockAddrBase::getIPv6Storage()
   {
     return (sockaddr_in6 *)&m_addr;
   }
 
   /**
-   * @warning does not checking at all 
+   * @warning does no checking at all
    */
-  inline sockaddr_in *sockAddrBase::getIPv4Storage() 
+  inline sockaddr_in* sockAddrBase::getIPv4Storage()
   {
     return (sockaddr_in *)&m_addr;
   }
 
   /**
-   * @warning does not checking at all 
+   * @warning does no checking at all
    */
-  inline const sockaddr_in6 *sockAddrBase::getIPv6Storage() const
+  inline const sockaddr_in6* sockAddrBase::getIPv6Storage() const
   {
     return (const sockaddr_in6 *)&m_addr;
   }
 
   /**
-   * @warning does not checking at all 
+   * @warning does no checking at all
    */
-  inline const sockaddr_in *sockAddrBase::getIPv4Storage() const
+  inline const sockaddr_in* sockAddrBase::getIPv4Storage() const
   {
     return (const sockaddr_in *)&m_addr;
   }
 
-  const char *sockAddrBase::ToString(bool includePort /*true*/) const
+  const char* sockAddrBase::ToString(bool includePort /*true*/) const
   {
-    in_port_t port = includePort ? Port():0;
+    in_port_t port = includePort ? Port() : 0;
 
     if (IsIPv4())
     {
@@ -410,14 +397,14 @@ namespace openbfdd
         // composite address
         char ifNameBuf[IF_NAMESIZE];
         char addrStrBuf[INET6_ADDRSTRLEN];
-        const char *ifName=NULL;
-        const char *addrStr=NULL;
+        const char *ifName = NULL;
+        const char *addrStr = NULL;
 
         if (storage->sin6_scope_id)
         {
-            ifName = if_indextoname(storage->sin6_scope_id, ifNameBuf);
-            if (!ifName)
-              ifName = "???";
+          ifName = if_indextoname(storage->sin6_scope_id, ifNameBuf);
+          if (!ifName)
+            ifName = "???";
         }
 
         addrStr = inet_ntop(AF_INET6, &storage->sin6_addr, addrStrBuf, sizeof(addrStrBuf));
@@ -438,34 +425,34 @@ namespace openbfdd
           else
           {
             LogVerify(false); // should never happen
-            return FormatShortStr("%s", addrStr);     
+            return FormatShortStr("%s", addrStr);
           }
         }
       }
-    }   
+    }
     else
       return "<Invalid Address>";
   }
 
-  /** 
-   * Compares two addresses. 
-   * Invalid addresses always compare the same, and below anything else. 
-   * IPv4 addresses always compare smaller than IPv6. 
-   * Assumes that IPv4 Addresses are in network order. 
-   *  
-   * If is IPv6 then comparison is: 
+  /**
+   * Compares two addresses.
+   * Invalid addresses always compare the same, and below anything else.
+   * IPv4 addresses always compare smaller than IPv6.
+   * Assumes that IPv4 Addresses are in network order.
+   *
+   * If is IPv6 then comparison is:
    *    sin6_addr
-   *    sin6_scope_id 
+   *    sin6_scope_id
    *    sin6_port
    *    sin6_flowinfo
-   *  
-   * If is IPv4 then comparison is: 
+   *
+   * If is IPv4 then comparison is:
    *    sin_addr
    *    sin_port
-   *  
-   * @param rhs 
-   * 
-   * @return: -1, 0, or 1 like strcmp.  
+   *
+   * @param rhs
+   *
+   * @return: -1, 0, or 1 like strcmp.
    */
   int sockAddrBase::compare(const sockAddrBase &rhs, bool comparePort) const
   {
@@ -492,9 +479,9 @@ namespace openbfdd
       {
         if (!comparePort || Port() == rhs.Port())
           return 0;
-        return Port() > rhs.Port() ? 1:-1;
+        return Port() > rhs.Port() ? 1 : -1;
       }
-      return ntohl(addrLhs) > ntohl(addrRhs) ? 1:-1;
+      return ntohl(addrLhs) > ntohl(addrRhs) ? 1 : -1;
     }
     else
     {
@@ -505,20 +492,19 @@ namespace openbfdd
       if (val != 0)
         return val;
       if (storageLhs->sin6_scope_id != storageRhs->sin6_scope_id)
-        return ntohl(storageLhs->sin6_scope_id) > ntohl(storageRhs->sin6_scope_id) ? 1:-1;
+        return ntohl(storageLhs->sin6_scope_id) > ntohl(storageRhs->sin6_scope_id) ? 1 : -1;
       if (comparePort && Port() != rhs.Port())
-        return Port() > rhs.Port() ? 1:-1;
+        return Port() > rhs.Port() ? 1 : -1;
       if (storageLhs->sin6_flowinfo != storageRhs->sin6_flowinfo)
-        return ntohl(storageLhs->sin6_flowinfo) > ntohl(storageRhs->sin6_flowinfo) ? 1:-1;
+        return ntohl(storageLhs->sin6_flowinfo) > ntohl(storageRhs->sin6_flowinfo) ? 1 : -1;
       return 0;
     }
   }
 
   /**
    * This needs a  to match compare.
-   * 
-   * 
-   * @return size_t 
+   *
+   * @return size_t
    */
   size_t sockAddrBase::hash() const
   {
@@ -533,7 +519,7 @@ namespace openbfdd
       // May not be the 'best' hash, but it should work. We add DNSP_HASHINIT
       // So that address 0.0.0.0 does not collide with !IsValid().
       if (!m_allowPort)
-        return getIPv4Storage()->sin_addr.s_addr+DNSP_HASHINIT;
+        return getIPv4Storage()->sin_addr.s_addr + DNSP_HASHINIT;
       else
       {
         const sockaddr_in *storage = getIPv4Storage();
@@ -547,31 +533,33 @@ namespace openbfdd
       // Below is to make sure that we can use hashword. This should never fail, but
       // without the 'if' some compilers issue a warning.
       if (!LogVerify((sizeof(in6_addr) % 4) == 0))
-        return 0; 
+        return 0;
 
       // May not be the 'best' hash, but it should work.
       uint32_t port = m_allowPort ? storage->sin6_port : 0;
-      uint32_t hash = hashword(reinterpret_cast<const uint32_t *>(&storage->sin6_addr), 
-                                sizeof(in6_addr)/4, DNSP_HASHINIT + port);
+      uint32_t hash = hashword(reinterpret_cast<const uint32_t *>(&storage->sin6_addr),
+                               sizeof(in6_addr) / 4, DNSP_HASHINIT + port);
       return (hash + storage->sin6_scope_id + storage->sin6_flowinfo);
     }
   }
 
   bool sockAddrBase::FromString(const char *str)
   {
+    // This is called from the constructor, so it must work with an uninitialized
+    // structure.
     init(Addr::Invalid);
     // First determine if it is IPv4 or IPv6
     str = SkipWhite(str);
-    const char *firstColon = strchr(str, ':'); 
-    const char *firstDot = strchr(str, '.'); 
-    const char *firstPercent = strchr(str, '%'); 
+    const char *firstColon = strchr(str, ':');
+    const char *firstDot = strchr(str, '.');
+    const char *firstPercent = strchr(str, '%');
 
     if (!firstDot && !firstColon)
       return false;
-    if (!firstColon 
+    if (!firstColon
         || (firstDot && firstDot < firstColon))
     {
-      // IPv4 
+      // IPv4
       uint16_t port;
       uint32_t addr;
 
@@ -587,7 +575,6 @@ namespace openbfdd
           return false;
         init(Addr::IPv4);
         getIPv4Storage()->sin_addr.s_addr = addr;
-        m_isValid = true;
         return true;
       }
       // Has dots and a colon, presumably its an IPv4 with port
@@ -598,7 +585,6 @@ namespace openbfdd
         return false;
       init(Addr::IPv4);
       getIPv4Storage()->sin_addr.s_addr = addr;
-      m_isValid = true;
       SetPort(port);
       return true;
     }
@@ -608,10 +594,10 @@ namespace openbfdd
       return false;
     else
     {
-      // IPv6 
+      // IPv6
       sockaddr_in6 tmp = sockaddr_in6();
 
-      if (firstPercent || *str == '[' )
+      if (firstPercent || *str == '[')
       {
         char addrStr[INET6_ADDRSTRLEN];
         const char *last, *lastBrace = NULL;
@@ -625,15 +611,15 @@ namespace openbfdd
         }
         if (firstPercent)
         {
-          if (lastBrace && lastBrace<firstPercent+2)
+          if (lastBrace && lastBrace < firstPercent + 2)
             return false;
           last = firstPercent;
         }
 
-        if (last==str || size_t(last-str) >= sizeof(addrStr))
+        if (last == str || size_t(last - str) >= sizeof(addrStr))
           return false;
-        memcpy(addrStr, str, last-str);
-        addrStr[last-str] = '\0';
+        memcpy(addrStr, str, last - str);
+        addrStr[last - str] = '\0';
         if (1 != inet_pton(AF_INET6, addrStr, &tmp.sin6_addr))
           return false;
         if (firstPercent)
@@ -647,22 +633,22 @@ namespace openbfdd
           if (lastBrace)
           {
             // already checked that (lastBrace-firstPercent >= 2)
-            memcpy(addrStr, firstPercent+1, lastBrace-(firstPercent+1));
-            addrStr[lastBrace-(firstPercent+1)] = '\0';
-            last = addrStr; 
+            memcpy(addrStr, firstPercent + 1, lastBrace - (firstPercent + 1));
+            addrStr[lastBrace - (firstPercent + 1)] = '\0';
+            last = addrStr;
           }
           else
-            last = firstPercent+1;
+            last = firstPercent + 1;
 
           unsigned int index = if_nametoindex(last);
-          if (index == 0 || index > UINT32_MAX )
+          if (index == 0 || index > UINT32_MAX)
             return false;
           tmp.sin6_scope_id = uint32_t(index);
         }
 
         if (lastBrace)
         {
-          last = lastBrace+1;
+          last = lastBrace + 1;
           if (*last != ':')
           {
             last = SkipWhite(last);
@@ -698,7 +684,6 @@ namespace openbfdd
       storage->sin6_addr = tmp.sin6_addr;
       storage->sin6_port = tmp.sin6_port;
       storage->sin6_scope_id = tmp.sin6_scope_id;
-      m_isValid = true;
       return true;
     }
   }
@@ -712,8 +697,4 @@ namespace openbfdd
       SetPort(port);
     return true;
   }
-}                                                                                
-
-
-
-
+}
